@@ -1,47 +1,68 @@
+import hashlib
+import logging
 import pkginfo
 import re
 import shutil
 import tempfile
+import zipfile
 import json
+from aiohttp.client_exceptions import ClientError
 from collections import defaultdict
+from datetime import timezone
 from django.conf import settings
+from django.db.utils import IntegrityError
 from jinja2 import Template
 from packaging.utils import canonicalize_name
 from packaging.requirements import Requirement
 from packaging.version import parse, InvalidVersion
-from pulpcore.plugin.models import Remote
+from pypi_simple import ACCEPT_JSON_PREFERRED, ProjectPage
+from pulpcore.plugin.models import Artifact, Remote
+from pulpcore.plugin.exceptions import TimeoutException
+from pulpcore.plugin.util import get_domain
+
+
+log = logging.getLogger(__name__)
 
 
 PYPI_LAST_SERIAL = "X-PYPI-LAST-SERIAL"
 """TODO This serial constant is temporary until Python repositories implements serials"""
 PYPI_SERIAL_CONSTANT = 1000000000
+SUPPORTED_METADATA_VERSIONS = ("1.0", "1.1", "1.2", "2.0", "2.1", "2.2", "2.3", "2.4")
+
+SIMPLE_API_VERSION = "1.1"
+PYPI_SIMPLE_V1_HTML = "application/vnd.pypi.simple.v1+html"
+PYPI_SIMPLE_V1_JSON = "application/vnd.pypi.simple.v1+json"
 
 simple_index_template = """<!DOCTYPE html>
 <html>
   <head>
     <title>Simple Index</title>
-    <meta name="api-version" value="2" />
+    <meta name="pypi:repository-version" content="{{ SIMPLE_API_VERSION }}">
   </head>
   <body>
     {% for name, canonical_name in projects %}
-    <a href="{{ canonical_name }}/">{{ name }}</a><br/>
+      <a href="{{ canonical_name }}/">{{ name }}</a><br/>
     {% endfor %}
   </body>
 </html>
 """
 
+# TODO in the future: data-requires-python (PEP 503)
 simple_detail_template = """<!DOCTYPE html>
 <html>
-<head>
-  <title>Links for {{ project_name }}</title>
-  <meta name="api-version" value="2" />
-</head>
-<body>
+  <head>
+    <title>Links for {{ project_name }}</title>
+    <meta name="pypi:repository-version" content="{{ SIMPLE_API_VERSION }}">
+  </head>
+  <body>
     <h1>Links for {{ project_name }}</h1>
-    {% for name, path, sha256 in project_packages %}
-    <a href="{{ path }}#sha256={{ sha256 }}" rel="internal">{{ name }}</a><br/>
-    {% endfor %}
-</body>
+    {%- for pkg in project_packages %}
+      <a href="{{ pkg.url }}#sha256={{ pkg.sha256 }}" rel="internal"
+      {%- if pkg.metadata_sha256 %} data-dist-info-metadata="sha256={{ pkg.metadata_sha256 }}"
+      {%- endif %} {% if pkg.provenance -%}
+      data-provenance="{{ pkg.provenance }}"{% endif %}>{{ pkg.filename }}</a><br/>
+    {%- endfor %}
+  </body>
 </html>
 """
 
@@ -128,6 +149,7 @@ def parse_project_metadata(project):
         # Release metadata
         "packagetype": project.get("packagetype") or "",
         "python_version": project.get("python_version") or "",
+        "metadata_sha256": project.get("metadata_sha256"),
     }
 
 
@@ -154,10 +176,9 @@ def parse_metadata(project, version, distribution):
     package["version"] = version
     package["url"] = distribution.get("url") or ""
     package["sha256"] = distribution.get("digests", {}).get("sha256") or ""
-    package["python_version"] = distribution.get("python_version") or package.get("python_version")
-    package["requires_python"] = distribution.get("requires_python") or package.get(
-        "requires_python"
-    )  # noqa: E501
+    package["python_version"] = distribution.get("python_version") or ""
+    package["requires_python"] = distribution.get("requires_python") or ""
+    package["size"] = distribution.get("size") or 0
 
     return package
 
@@ -175,6 +196,7 @@ def get_project_metadata_from_file(filename):
     packagetype = DIST_EXTENSIONS[extensions[pkg_type_index]]
 
     metadata = DIST_TYPES[packagetype](filename)
+    metadata.metadata_sha256 = compute_metadata_sha256(filename)
     metadata.packagetype = packagetype
     if packagetype == "sdist":
         metadata.python_version = "source"
@@ -187,6 +209,34 @@ def get_project_metadata_from_file(filename):
     return metadata
 
 
+def extract_wheel_metadata(filename: str) -> bytes | None:
+    """
+    Extract the metadata file content from a wheel file.
+
+    Returns the raw metadata content as bytes or None if metadata cannot be extracted.
+    """
+    if not filename.endswith(".whl"):
+        return None
+    try:
+        with zipfile.ZipFile(filename, "r") as f:
+            for file_path in f.namelist():
+                if file_path.endswith(".dist-info/METADATA"):
+                    return f.read(file_path)
+    except (zipfile.BadZipFile, KeyError, OSError) as e:
+        log.warning(f"Failed to extract metadata file from {filename}: {e}")
+    return None
+
+
+def compute_metadata_sha256(filename: str) -> str | None:
+    """
+    Compute SHA256 hash of the metadata file from a Python package.
+
+    Returns SHA256 hash or None if metadata cannot be extracted.
+    """
+    metadata_content = extract_wheel_metadata(filename)
+    return hashlib.sha256(metadata_content).hexdigest() if metadata_content else None
+
+
 def artifact_to_python_content_data(filename, artifact, domain=None):
     """
     Takes the artifact/filename and returns the metadata needed to create a PythonPackageContent.
@@ -195,17 +245,55 @@ def artifact_to_python_content_data(filename, artifact, domain=None):
     # because pkginfo validates that the filename has a valid extension before
     # reading it
     with tempfile.NamedTemporaryFile("wb", dir=".", suffix=filename) as temp_file:
+        artifact.file.seek(0)
         shutil.copyfileobj(artifact.file, temp_file)
         temp_file.flush()
         metadata = get_project_metadata_from_file(temp_file.name)
     data = parse_project_metadata(vars(metadata))
     data["sha256"] = artifact.sha256
+    data["size"] = artifact.size
     data["filename"] = filename
     data["pulp_domain"] = domain or artifact.pulp_domain
     data["_pulp_domain"] = data["pulp_domain"]
     return data
 
 
+def artifact_to_metadata_artifact(
+    filename: str, artifact: Artifact, tmp_dir: str = "."
+) -> Artifact | None:
+    """
+    Creates artifact for metadata from the provided wheel artifact.
+    """
+    if not filename.endswith(".whl"):
+        return None
+
+    with tempfile.NamedTemporaryFile("wb", dir=tmp_dir, suffix=filename, delete=False) as temp_file:
+        temp_wheel_path = temp_file.name
+        artifact.file.seek(0)
+        shutil.copyfileobj(artifact.file, temp_file)
+        temp_file.flush()
+
+    metadata_content = extract_wheel_metadata(temp_wheel_path)
+    if not metadata_content:
+        return None
+
+    with tempfile.NamedTemporaryFile(
+        "wb", dir=tmp_dir, suffix=".metadata", delete=False
+    ) as temp_md:
+        temp_metadata_path = temp_md.name
+        temp_md.write(metadata_content)
+        temp_md.flush()
+
+    metadata_artifact = Artifact.init_and_validate(temp_metadata_path)
+    try:
+        metadata_artifact.save()
+    except IntegrityError:
+        metadata_artifact = Artifact.objects.get(
+            sha256=metadata_artifact.sha256, pulp_domain=get_domain()
+        )
+    return metadata_artifact
+
+
 def fetch_json_release_metadata(name: str, version: str, remotes: set[Remote]) -> dict:
     """
     Fetches metadata for a specific release from PyPI's JSON API. A release can contain
@@ -369,7 +457,9 @@ def python_content_to_download_info(content, base_path, domain=None):
             _art = models.RemoteArtifact.objects.filter(content_artifact=content_artifact).first()
         return _art
 
-    content_artifact = content.contentartifact_set.first()
+    content_artifact = content.contentartifact_set.exclude(
+        relative_path__endswith=".metadata"
+    ).first()
     artifact = find_artifact()
     origin = settings.CONTENT_ORIGIN or settings.PYPI_API_HOSTNAME or ""
     origin = origin.strip("/")
@@ -380,7 +470,6 @@ def python_content_to_download_info(content, base_path, domain=None):
         components.insert(2, domain.name)
     url = "/".join(components)
     md5 = artifact.md5 if artifact and artifact.md5 else ""
-    size = artifact.size if artifact and artifact.size else 0
     return {
         "comment_text": "",
         "digests": {"md5": md5, "sha256": content.sha256},
@@ -391,7 +480,7 @@ def python_content_to_download_info(content, base_path, domain=None):
         "packagetype": content.packagetype,
         "python_version": content.python_version,
         "requires_python": content.requires_python or None,
-        "size": size,
+        "size": content.size,
         "upload_time": str(content.pulp_created),
         "upload_time_iso_8601": str(content.pulp_created.isoformat()),
         "url": url,
@@ -403,17 +492,78 @@ def python_content_to_download_info(content, base_path, domain=None):
 def write_simple_index(project_names, streamed=False):
     """Writes the simple index."""
     simple = Template(simple_index_template)
-    context = {"projects": ((x, canonicalize_name(x)) for x in project_names)}
+    context = {
+        "SIMPLE_API_VERSION": SIMPLE_API_VERSION,
+        "projects": ((x, canonicalize_name(x)) for x in project_names),
+    }
     return simple.stream(**context) if streamed else simple.render(**context)
 
 
 def write_simple_detail(project_name, project_packages, streamed=False):
     """Writes the simple detail page of a package."""
     detail = Template(simple_detail_template)
-    context = {"project_name": project_name, "project_packages": project_packages}
+    context = {
+        "SIMPLE_API_VERSION": SIMPLE_API_VERSION,
+        "project_name": project_name,
+        "project_packages": project_packages,
+    }
     return detail.stream(**context) if streamed else detail.render(**context)
 
 
+def write_simple_index_json(project_names):
+    """Writes the simple index in JSON format."""
+    return {
+        "meta": {"api-version": SIMPLE_API_VERSION, "_last-serial": PYPI_SERIAL_CONSTANT},
+        "projects": [
+            {"name": name, "_last-serial": PYPI_SERIAL_CONSTANT} for name in project_names
+        ],
+    }
+
+
+def write_simple_detail_json(project_name, project_packages):
+    """Writes the simple detail page in JSON format."""
+    return {
+        "meta": {"api-version": SIMPLE_API_VERSION, "_last-serial": PYPI_SERIAL_CONSTANT},
+        "name": canonicalize_name(project_name),
+        "files": [
+            {
+                # v1.0, PEP 691
+                "filename": package["filename"],
+                "url": package["url"],
+                "hashes": {"sha256": package["sha256"]},
+                "requires-python": package["requires_python"] or None,
+                # data-dist-info-metadata is deprecated alias for core-metadata
+                "data-dist-info-metadata": (
+                    {"sha256": package["metadata_sha256"]} if package["metadata_sha256"] else False
+                ),
+                # yanked and yanked_reason are not implemented because they are mutable
+                # (v1.1, PEP 700)
+                "size": package["size"],
+                "upload-time": format_upload_time(package["upload_time"]),
+                # TODO in the future:
+                # core-metadata (PEP 7.14)
+                # (v1.3, PEP 740)
+                "provenance": package.get("provenance", None),
+            }
+            for package in project_packages
+        ],
+        # (v1.1, PEP 700)
+        "versions": sorted(set(package["version"] for package in project_packages)),
+        # TODO in the future:
+        # alternate-locations (v1.2, PEP 708)
+        # project-status (v1.4, PEP 792 - pypi and docs differ)
+    }
+
+
+def format_upload_time(upload_time):
+    """Formats the upload time to be in Zulu time. UTC with Z suffix"""
+    if upload_time:
+        if upload_time.tzinfo:
+            dt = upload_time.astimezone(timezone.utc)
+            return dt.isoformat().replace("+00:00", "Z")
+    return None
+
+
 class PackageIncludeFilter:
     """A special class to help filter Package's based on a remote's include/exclude"""
 
@@ -489,3 +639,39 @@ def get_remote_package_filter(remote):
     rfilter = PackageIncludeFilter(remote)
     _remote_filters[remote.pulp_id] = (remote.pulp_last_updated, rfilter)
     return rfilter
+
+
+def get_remote_simple_page(package, remote, max_retries=1):
+    """Gets the simple page for a package from a remote."""
+    url = remote.get_remote_artifact_url(f"simple/{package}/")
+    remote.headers = remote.headers or []
+    remote.headers.append({"Accept": ACCEPT_JSON_PREFERRED})
+    downloader = remote.get_downloader(url=url, max_retries=max_retries)
+    try:
+        d = downloader.fetch()
+    except (ClientError, TimeoutException):
+        return None
+
+    if d.headers["content-type"] == PYPI_SIMPLE_V1_JSON:
+        page = ProjectPage.from_json_data(json.load(open(d.path, "rb")), base_url=url)
+    else:
+        page = ProjectPage.from_html(package, open(d.path, "rb").read(), base_url=url)
+    return page
+
+
+async def aget_remote_simple_page(package, remote, max_retries=1):
+    """Gets the simple page for a package from a remote."""
+    url = remote.get_remote_artifact_url(f"simple/{package}/")
+    remote.headers = remote.headers or []
+    remote.headers.append({"Accept": ACCEPT_JSON_PREFERRED})
+    downloader = remote.get_downloader(url=url, max_retries=max_retries)
+    try:
+        d = await downloader.run()
+    except (ClientError, TimeoutException):
+        return None
+
+    if d.headers["content-type"] == PYPI_SIMPLE_V1_JSON:
+        page = ProjectPage.from_json_data(json.load(open(d.path, "rb")), base_url=url)
+    else:
+        page = ProjectPage.from_html(package, open(d.path, "rb").read(), base_url=url)
+    return page
