@@ -5,39 +5,19 @@ import subprocess
 from urllib.parse import urljoin
 
 from pulp_python.tests.functional.constants import (
-    PYTHON_SM_PROJECT_SPECIFIER,
-    PYTHON_SM_FIXTURE_RELEASES,
-    PYTHON_SM_FIXTURE_CHECKSUMS,
+    PYPI_SERIAL_CONSTANT,
     PYTHON_MD_PROJECT_SPECIFIER,
     PYTHON_MD_PYPI_SUMMARY,
     PYTHON_EGG_FILENAME,
-    PYTHON_EGG_URL,
     PYTHON_EGG_SHA256,
     PYTHON_WHEEL_FILENAME,
-    PYTHON_WHEEL_URL,
     PYTHON_WHEEL_SHA256,
     SHELF_PYTHON_JSON,
 )
-
-from pulp_python.tests.functional.utils import ensure_simple
+from pulp_python.tests.functional.utils import ensure_metadata
 
 
 PYPI_LAST_SERIAL = "X-PYPI-LAST-SERIAL"
-PYPI_SERIAL_CONSTANT = 1000000000
-
-
-@pytest.fixture
-def python_empty_repo_distro(python_repo_factory, python_distribution_factory):
-    """Returns an empty repo with and distribution serving it."""
-
-    def _generate_empty_repo_distro(repo_body=None, distro_body=None):
-        repo_body = repo_body or {}
-        distro_body = distro_body or {}
-        repo = python_repo_factory(**repo_body)
-        distro = python_distribution_factory(repository=repo, **distro_body)
-        return repo, distro
-
-    yield _generate_empty_repo_distro
 
 
 @pytest.mark.parallel
@@ -80,19 +60,6 @@ def test_published_index(
     assert summary.to_dict() == PYTHON_MD_PYPI_SUMMARY
 
 
-@pytest.fixture(scope="module")
-def python_package_dist_directory(tmp_path_factory, http_get):
-    """Creates a temp dir to hold package distros for uploading."""
-    dist_dir = tmp_path_factory.mktemp("dist")
-    egg_file = dist_dir / PYTHON_EGG_FILENAME
-    wheel_file = dist_dir / PYTHON_WHEEL_FILENAME
-    with open(egg_file, "wb") as f:
-        f.write(http_get(PYTHON_EGG_URL))
-    with open(wheel_file, "wb") as f:
-        f.write(http_get(PYTHON_WHEEL_URL))
-    yield dist_dir, egg_file, wheel_file
-
-
 @pytest.mark.parallel
 def test_package_upload(
     python_content_summary, python_empty_repo_distro, python_package_dist_directory, monitor_task
@@ -172,6 +139,35 @@ def test_package_upload_simple(
     assert summary.added["python.python"]["count"] == 1
 
 
+@pytest.mark.parallel
+def test_package_upload_with_metadata(
+    monitor_task,
+    pulp_content_url,
+    python_content_summary,
+    python_empty_repo_distro,
+    python_package_dist_directory,
+):
+    """
+    Test that the upload of a Python wheel package creates a metadata artifact.
+    """
+    repo, distro = python_empty_repo_distro()
+    url = urljoin(distro.base_url, "simple/")
+    dist_dir, egg_file, wheel_file = python_package_dist_directory
+    response = requests.post(
+        url,
+        data={"sha256_digest": PYTHON_WHEEL_SHA256},
+        files={"content": open(wheel_file, "rb")},
+        auth=("admin", "password"),
+    )
+    assert response.status_code == 202
+    monitor_task(response.json()["task"])
+    summary = python_content_summary(repository=repo)
+    assert summary.added["python.python"]["count"] == 1
+
+    # Test that metadata is accessible
+    ensure_metadata(pulp_content_url, distro.base_path, PYTHON_WHEEL_FILENAME)
+
+
 @pytest.mark.parallel
 def test_twine_upload(
     pulpcore_bindings,
@@ -225,26 +221,6 @@ def test_twine_upload(
             check=True,
         )
 
-    # Test re-uploading same packages with --skip-existing works
-    output = subprocess.run(
-        (
-            "twine",
-            "upload",
-            "--repository-url",
-            url,
-            dist_dir / "*",
-            "-u",
-            username,
-            "-p",
-            password,
-            "--skip-existing",
-        ),
-        capture_output=True,
-        check=True,
-        text=True,
-    )
-    assert output.stdout.count("Skipping") == 2
-
 
 @pytest.mark.parallel
 def test_simple_redirect_with_publications(
@@ -263,22 +239,6 @@ def test_simple_redirect_with_publications(
     assert response.url == str(urljoin(pulp_content_url, f"{distro.base_path}/simple/"))
 
 
-@pytest.mark.parallel
-def test_simple_correctness_live(
-    python_remote_factory, python_repo_with_sync, python_distribution_factory
-):
-    """Checks that the simple api on live distributions are correct."""
-    remote = python_remote_factory(includes=PYTHON_SM_PROJECT_SPECIFIER)
-    repo = python_repo_with_sync(remote)
-    distro = python_distribution_factory(repository=repo)
-    proper, msgs = ensure_simple(
-        urljoin(distro.base_url, "simple/"),
-        PYTHON_SM_FIXTURE_RELEASES,
-        sha_digests=PYTHON_SM_FIXTURE_CHECKSUMS,
-    )
-    assert proper is True, msgs
-
-
 @pytest.mark.parallel
 def test_pypi_json(python_remote_factory, python_repo_with_sync, python_distribution_factory):
     """Checks the data of `pypi/{package_name}/json` endpoint."""
@@ -287,6 +247,11 @@ def test_pypi_json(python_remote_factory, python_repo_with_sync, python_distribu
     distro = python_distribution_factory(repository=repo)
     response = requests.get(urljoin(distro.base_url, "pypi/shelf-reader/json"))
     assert_pypi_json(response.json())
+    # Test serving a repository version
+    distro = python_distribution_factory(repository=repo, version="1")
+    assert distro.repository is None
+    response = requests.get(urljoin(distro.base_url, "pypi/shelf-reader/json"))
+    assert_pypi_json(response.json())
 
 
 @pytest.mark.parallel
