@@ -112,19 +112,22 @@ PYTHON_LG_PROJECT_SPECIFIER = [
     "aiohttp",  # matches 7
     "bcrypt",  # matches 8
     "celery",  # matches 13
+    "crane",  # matches 0
     "Django",  # matches 31
+    "pulp-2to3-migration",  # matches 2
     "pytz",  # matches 6
     "scipy",  # matches 23
     "setuptools",  # matches 2
     "shelf-reader",  # matches 2
+    "twine",  # matches 14
 ]
-PYTHON_LG_PACKAGE_COUNT = 92
+PYTHON_LG_PACKAGE_COUNT = 108
 PYTHON_LG_FIXTURE_SUMMARY = {PYTHON_CONTENT_NAME: PYTHON_LG_PACKAGE_COUNT}
 PYTHON_LG_FIXTURE_COUNTS = {
-    "latest_3": 51,
-    "sdist": 28,
-    "bdist_wheel": 64,
-    "multi": 34,  # keep_latest=1, package_types="bdist_wheel", prereleases=False
+    "latest_3": 59,
+    "sdist": 36,
+    "bdist_wheel": 72,
+    "multi": 36,  # keep_latest=1, package_types="bdist_wheel", prereleases=False
 }
 
 DJANGO_LATEST_3 = 4  # latest version has 2 dists, each other has 1
@@ -147,6 +150,8 @@ PYTHON_WHEEL_FILENAME = "shelf_reader-0.1-py2-none-any.whl"
 PYTHON_WHEEL_URL = urljoin(urljoin(PYTHON_FIXTURES_URL, "packages/"), PYTHON_WHEEL_FILENAME)
 PYTHON_WHEEL_SHA256 = "2eceb1643c10c5e4a65970baf63bde43b79cbdac7de81dae853ce47ab05197e9"
 
+PYTHON_WHEEL_METADATA_SHA256 = "ed333f0db05d77e933a157b7225b403ada9a2f93318d77b41b662eba78bac350"
+
 PYTHON_XS_FIXTURE_CHECKSUMS = {
     PYTHON_EGG_FILENAME: PYTHON_EGG_SHA256,
     PYTHON_WHEEL_FILENAME: PYTHON_WHEEL_SHA256,
@@ -350,3 +355,5 @@ VULNERABILITY_REPORT_TEST_PACKAGE_NAME = "Django"
 VULNERABILITY_REPORT_TEST_PACKAGES = [
     "django==5.2.1",
 ]
+
+PYPI_SERIAL_CONSTANT = 1000000000
